24913be84f
* cmake formatting (cmake-format) (#188)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* source formatting (clang-format v11) (#189)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: design of the pc sampling data struct; guarding parts of code that uses ROCr marker packets
* source formatting (clang-format v11) (#191)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* cmake formatting (cmake-format) (#192)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: shadow variable fix
* pcs: fix for compiler errors reported by CI/CD
* source formatting (clang-format v11) (#193)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: docs fix; samples uses rocprofiler::rocprofiler library
* cmake formatting (cmake-format) (#195)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: client in samples folder fixed
* pcs: client requires rocprofiler package as dependency
* pcs: client uses single context
* source formatting (clang-format v11) (#196)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: client using single buffer; no buffer destroy in client
* pcs: client::setup explicitly called from the example
* pcs: rocprofiler_pc_sample_record_t updated
* pcs: fixed init of external correlation id
* source formatting (clang-format v11) (#198)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: remove outdated files; update CMakeLists
* cmake formatting (cmake-format) (#212)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: using rocprofiler_agent_id_t
* pcs: Removing trailing whitespaces
Co-authored-by: Jonathan R. Madsen <jrmadsen@users.noreply.github.com>
* source formatting (clang-format v11) (#214)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: mapping agent_id to the agent
* source formatting (clang-format v11) (#215)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: const while iterating over agents
* source formatting (clang-format v11) (#216)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: calling get_buffer instead of get_buffers
* pcs: workgroup typo
* pcs: documentation for the public PC sampling API
* pcs: queue_cb_t signature adaptation
* pcs: mocks removed
* pcs: updating HsaApiTable with HSA/ROCr PC sampling API
* pcs: querying available PC sampling configs through IOCTL
* pcs: create the PCS session in IOCTL
* pcs: first actual PC samples delivered to the rocprofiler's client :)
* pcs: works with marker packet too
* pcs: using HSA table to call pc sampling related functions
* pcs: using ioctl instead of kfd in naming
* pcs: configuration service test fixed
* pcs: sample processing test fixed
* pcs: marker packet macro wrapper removed
* pcs: marker packet is part of the rocprofiler_packet union
* pcs: one fixme added
* pcs: client that uses pc-sampling and code obj tracing
* pcs: client that supprts PC sampling and code obj tracing refactored
* pcs: show more info for each PC sample
* pcs: hex output for the samples that do not belong to the matmul kernel
* pcs: querying avail configuration happens immediately before configuring
* pcs: hsa_ven_amd_pcs_create_from_id renamed
* pcs: using hsa_stop; accessing a buffer by id from parser
* pcs: includes reworked, tests returned to life
* pcs: rocrofiler dir removed as outdated
* cmake formatting (cmake-format) (#271)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* source formatting (clang-format v11) (#272)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: some warnings fixed
* source formatting (clang-format v11) (#273)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* cmake formatting (cmake-format) (#274)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: show MI200 relevant information in the sample
* pcs: queue cb fixed; rocr.h include fixed
* source formatting (clang-format v11) (#296)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: getting hsa_agent and the doorbell_id from hsa_queue
* source formatting (clang-format v11) (#297)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: correlation ID logic fixed
* source formatting (clang-format v11) (#303)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: pure pc sampling example fixed
* source formatting (clang-format v11) (#307)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* cmake formatting (cmake-format) (#308)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: interval value if the PC sampling is already configured
* pcs: ROCPROFILER_STATUS_ERROR_PC_SAMPLING_ALREADY_CONFIGURED
New status code if another process configured PC sampling service with different configuration.
Samples are extended to consider this case and retry if it happens.
* pcs: hsa_amd_queue_get_info mocked in tests
* source formatting (clang-format v11) (#328)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs (tests): query configs after configuring service
* source formatting (clang-format v11) (#329)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: sample checks workgroup_id_* and wave_id
* source formatting (clang-format v11) (#330)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs samples: running samples on the device 0
* pcs: kfd_ioctl updated
* pcs: ioctl config struct changed fields names
* pcs: status when PC sampling is configured by another process is renamed
* pcs: HSA PC sampling API table fixed
* pcs: tmp hack to be able to use HSA pc sampling table
* source formatting (clang-format v11) (#443)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs service use CIDs generated by HIP API tracing service
* source formatting (clang-format v11) (#455)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* cmake formatting (cmake-format) (#456)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: CID manager
* pcs: explicit flush with no delivered data executes retirement logic
* source formatting (clang-format v11) (#464)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: rocprofiler_query_pc_sampling_agent_configurations docs update
* source formatting (clang-format v11) (#465)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: rocprofiler_configure_pc_sampling_service docs update
* pcs: explicit sync introduced in PCSCIDManager
* pcs: new logic for retiring CIDs in PC sampling service documented
* pcs: queue interception cb signature updated
* source formatting (clang-format v11) (#471)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: if no agents supports PC sampling, fail gracefully
* elaborating when KFD returns EBUSY and EEXIST
* pcs: the second PC sampling examples fails gracefully
* code samples use only single kernel for now
* pcs: CID manager refactored
* source formatting (clang-format v11) (#481)
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
* pcs: ioctl update
* source formatting (clang-format v11) (#531)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs:code sample to test PC sampling applied on concurrent kernels
* source formatting (clang-format v11) (#533)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: pc sampling strest test included
* cmake formatting (cmake-format) (#539)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* source formatting (clang-format v11) (#540)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: standalone benchmark
* cmake formatting (cmake-format) (#555)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: glance in external correlation IDs
* source formatting (clang-format v11) (#557)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* another change in ioctl interface
* pcs: update queue interceptor callbacks and samples accroding to the agent 0 version
* source formatting (clang-format v11) (#611)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: avoid running problematic PC sampling test
* pcs: guarding tests not to fail on architectures not supporting PC sampling
* source formatting (clang-format v11) (#617)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: check IOCTL version prior to each KFD call
* pcs: ioctl refactoring
* pcs: PC sampling service increases the ref_count of the correlation ID of the kernel dispatch
* cmake formatting (cmake-format) (#631)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* source formatting (clang-format v11) (#632)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: PC sampling service provides external correlation IDs
* source formatting (clang-format v11) (#644)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: use rocprofiler_dim3_t for workgrou_ip
* source formatting (clang-format v11) (#645)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: minor fixes
* pcs: updating the documentation for the pc sampling API functions
* pcs: api table and queue controller fix
* pcs: don't generate marker packets for the agent if PC sampling is not configured on it
* pcs: multi-GPU and single-GPU clients
* source formatting (clang-format v11) (#700)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: warning and errors fixed
* source formatting (clang-format v11) (#702)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: clang compiler errors and warnings fixed
* source formatting (clang-format v11) (#716)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: const reference in cid manager
* source formatting (clang-format v11) (#717)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: const & func in manager explicit
* pcs: test to cover creating PC sampling service of agent that does not exist
* pcs: generate marker packets if service is active
* source formatting (clang-format v11) (#719)
Co-authored-by: vlaindic <139573562+vlaindic@users.noreply.github.com>
* pcs: refactoring hsa_adapter; use the correlation_id->thread_idx
* Update source/lib/rocprofiler-sdk/pc_sampling/cid_manager.cpp
* Update source/lib/rocprofiler-sdk/pc_sampling/cid_manager.cpp
* Update source/lib/rocprofiler-sdk/pc_sampling/hsa_adapter.cpp
* Update source/lib/rocprofiler-sdk/pc_sampling/hsa_adapter.cpp
* Update source/lib/rocprofiler-sdk/pc_sampling/hsa_adapter.cpp
* Update source/lib/rocprofiler-sdk/pc_sampling/hsa_adapter.cpp
* Update source/lib/rocprofiler-sdk/pc_sampling/utils.cpp
* Update utils.cpp
* moving pc-sampling tests and samples to pc-sampling label
* Format fix
* pcs: use configured instead of active service
* Update source/lib/rocprofiler-sdk/pc_sampling/service.cpp
* pcs: ensure configuring PC sampling on the HSA level is called only once
* pcs: minor fix
* Update CMakeLists.txt
* Update CMakeLists.txt
* Update CMakeLists.txt
* Update CMakeLists.txt
* pcs: refactoring IOCTL integration
* Update source/lib/rocprofiler-sdk/pc_sampling/tests/CMakeLists.txt
Co-authored-by: Ammar ELWazir <ammar.elwazir@amd.com>
* Update source/lib/rocprofiler-sdk/pc_sampling/ioctl/ioctl_adapter.cpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* Update source/lib/rocprofiler-sdk/pc_sampling/ioctl/ioctl_adapter_types.hpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* Update source/lib/rocprofiler-sdk/pc_sampling/ioctl/ioctl_adapter.cpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* Update source/lib/rocprofiler-sdk/pc_sampling/ioctl/ioctl_adapter_types.hpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* Update source/lib/rocprofiler-sdk/pc_sampling/ioctl/ioctl_adapter.hpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* pcs: reverting back what bot doubled
* Update source/lib/rocprofiler-sdk/pc_sampling/ioctl/ioctl_adapter_types.hpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* pcs: retesting the bot
* Update source/lib/rocprofiler-sdk/pc_sampling/ioctl/ioctl_adapter_types.hpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* pcs: why bot fails on this IOCTL status
* pcs: why failing on <vector>
* Update source/lib/rocprofiler-sdk/pc_sampling/ioctl/ioctl_adapter.cpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* pcs: returning commits removed by bot
* pcs: formatting locally
* pcs: clients are flushing buffers inside the tool_fini
* pcs: sync function in public API
* pcs: sync prior to unloading the code object
* pcs: sync function requires context
* pcs: client uses CID retirement service
* pcs: test for flusing internal ROCr buffers
* pcs: source formatting
* Update source/lib/rocprofiler-sdk/pc_sampling/tests/CMakeLists.txt
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* pcs: code samples refactoring
* pcs: public API header refactored
* pcs: rocprofiler_buffer_flush drains internal PC sampling buffers too
* pcs: remove unnecessary functions
* pcs: do not call hsa's copytables
* pcs: include reordering
* pcs: using ROCP_ERROR inside PC sampling implementation
* pcs: pc_sampling sample uses ostream instean of printfs
* pcs: pc_sampling_codeobj tracing using ostream instead of prints
* pcs: registering once for interceptor callbacks
* pcs: do not generate internal CIDs if not in debug mode
* pcs: rebasing fixed; missing external correlation IDs
* pcs: code formatting
* enable kernel tracing service to receive external correlation IDs
* pcs: using ROCPROFILER_STATUS_ERROR_INCOMPATIBLE_KERNEL
* pcs: polishing parser
* formatting
* updating parser to use workgroup_id
* kfd_ioctl.h extracted in details folder
* refactoring
* pcs: preparing to generate code object information
* flush internal buffers prior to unloading code object
* pcs: generating marker records
* pcs: wrap code_object's shutdown function
* ROCR_VISIBLE_DEVICES and HIP_VISISBLE_DEVICES unsupported at the moment
* documenting the ignorance of ROCR/HIP_VISIBLE_DEVICES
* pcs: separate structs for code object loading/unloading markers
* pcs: inst_pkt_t changed the namespace
* pcs: removing wrapper around the shutdown function
* pcs: size in record field
* pcs: documentation refactoring + typdefs
* renaming PCSAgentConfig to PCSAgentSession
* pcs: service does not keep a pointer to the context
* pcs: static assertions related to the versioning
* pcs: rocprofiler_pc_sampling_configuration_t size field
* pcs: report API unimplemented unleass explicitly enabled
* pcs: skip tests if KFD does not support PC sampling
* pcs: if ROCr hides some devices, no PC samples will be delivered for it
* pcs: hip error check after kernel launch
* formatting
* removing PCS info from agent.h
* fix based on review
* Update continuous integration workflow
- use mi200 runner for code coverage (supports PC sampling)
- split sanitizer jobs across navi3, vega20, and mi300
* Updating pc sampling test labels
* ROCP_PC_SAMPLING_ENABLED env in CI
* ROCP_PC_SAMPLING_ENABLED for all CI mi200 jobs
* Rearrange sanitizer assignments
* fixes according to review
* removed unused functions
* pcs: rocprofiler_agent_id_t instead of handle as a key in map
* Update source/lib/rocprofiler-sdk/context/context.hpp
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
* removing drm_fd from the agent.h
* pcs: removing one sample due to complexity
* pcs: refactoring sample
* simplifying sample
* new lines
* Improve queue_control enable intercepter logic
* Update lib/rocprofiler-sdk/hsa/types.hpp
- handle amd_ext size for HSA 1.12.0
* ROCP_PC_SAMPLING_ENABLED -> ROCPROFILER_PC_SAMPLING_BETA_ENABLED
* Update hsa_adapter.cpp
- anonymous namespace + remove debug
* parser update
* Apply suggestions from code review
---------
Co-authored-by: github-actions[bot] <41898282+github-actions[bot]@users.noreply.github.com>
Co-authored-by: vlaindic <vlaindic@users.noreply.github.com>
Co-authored-by: vlaindic <vladimir.indic@amd.com>
Co-authored-by: vlaindic <vlaindic@amd.com>
Co-authored-by: Vladimir Indic <139573562+vlaindic@users.noreply.github.com>
Co-authored-by: Jonathan R. Madsen <jrmadsen@users.noreply.github.com>
Co-authored-by: gobhardw <gopesh.bhardwaj@amd.com>
Co-authored-by: Jonathan R. Madsen <jonathanrmadsen@gmail.com>
[ROCm/rocprofiler-sdk commit: 987ae3cc47]
413 líneas
13 KiB
C++
413 líneas
13 KiB
C++
// MIT License
|
|
//
|
|
// Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
|
|
//
|
|
// Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
// of this software and associated documentation files (the "Software"), to deal
|
|
// in the Software without restriction, including without limitation the rights
|
|
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
// copies of the Software, and to permit persons to whom the Software is
|
|
// furnished to do so, subject to the following conditions:
|
|
//
|
|
// The above copyright notice and this permission notice shall be included in
|
|
// all copies or substantial portions of the Software.
|
|
//
|
|
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
|
// THE SOFTWARE.
|
|
|
|
#include "hip/hip_runtime.h"
|
|
|
|
#include <chrono>
|
|
#include <cstdio>
|
|
#include <cstdlib>
|
|
#include <iostream>
|
|
#include <mutex>
|
|
#include <random>
|
|
#include <sstream>
|
|
#include <stdexcept>
|
|
|
|
#define HIP_API_CALL(CALL) \
|
|
{ \
|
|
hipError_t error_ = (CALL); \
|
|
if(error_ != hipSuccess) \
|
|
{ \
|
|
auto _hip_api_print_lk = auto_lock_t{print_lock}; \
|
|
fprintf(stderr, \
|
|
"%s:%d :: HIP error : %s\n", \
|
|
__FILE__, \
|
|
__LINE__, \
|
|
hipGetErrorString(error_)); \
|
|
throw std::runtime_error("hip_api_call"); \
|
|
} \
|
|
}
|
|
|
|
namespace
|
|
{
|
|
using auto_lock_t = std::unique_lock<std::mutex>;
|
|
auto print_lock = std::mutex{};
|
|
size_t nthread_per_device = 2;
|
|
size_t nitr = 500;
|
|
size_t nsync = 10;
|
|
constexpr unsigned shared_mem_tile_dim = 32;
|
|
|
|
void
|
|
check_hip_error(void);
|
|
|
|
void
|
|
verify(int* in, int* out, int M, int N);
|
|
} // namespace
|
|
|
|
__global__ void
|
|
transpose(const int* in, int* out, int M, int N);
|
|
|
|
void
|
|
run(int rank, int tid, int devid, int argc, char** argv);
|
|
|
|
void
|
|
run_transpose(int rank, int tid, hipStream_t stream, int argc, char** argv);
|
|
|
|
void
|
|
run_migrate(int rank, int tid, hipStream_t stream, int, char** argv);
|
|
|
|
void
|
|
run_scratch(int rank, int tid, hipStream_t stream, int argc, char** argv);
|
|
|
|
int
|
|
main(int argc, char** argv)
|
|
{
|
|
auto* exe_name = ::basename(argv[0]);
|
|
|
|
int rank = 0;
|
|
for(int i = 1; i < argc; ++i)
|
|
{
|
|
auto _arg = std::string{argv[i]};
|
|
if(_arg == "?" || _arg == "-h" || _arg == "--help")
|
|
{
|
|
fprintf(stderr,
|
|
"usage: %s [NUM_THREADS_PER_DEVICE (%zu)] [NUM_ITERATION (%zu)] "
|
|
"[SYNC_EVERY_N_ITERATIONS (%zu)]\n",
|
|
exe_name,
|
|
nthread_per_device,
|
|
nitr,
|
|
nsync);
|
|
exit(EXIT_SUCCESS);
|
|
}
|
|
}
|
|
if(argc > 1) nthread_per_device = atoll(argv[1]);
|
|
if(argc > 2) nitr = atoll(argv[2]);
|
|
if(argc > 3) nsync = atoll(argv[3]);
|
|
|
|
int ndevice = 0;
|
|
HIP_API_CALL(hipGetDeviceCount(&ndevice));
|
|
|
|
auto nthreads = (ndevice * nthread_per_device);
|
|
|
|
printf("[%s] Number of devices found: %i\n", exe_name, ndevice);
|
|
printf("[%s] Number of threads (per device): %zu\n", exe_name, nthread_per_device);
|
|
printf("[%s] Number of threads (total): %zu\n", exe_name, nthreads);
|
|
printf("[%s] Number of iterations: %zu\n", exe_name, nitr);
|
|
printf("[%s] Syncing every %zu iterations\n", exe_name, nsync);
|
|
|
|
{
|
|
auto _threads = std::vector<std::thread>{};
|
|
for(size_t i = 0; i < nthreads; ++i)
|
|
_threads.emplace_back(run, rank, i, i % ndevice, argc, argv);
|
|
for(auto& itr : _threads)
|
|
itr.join();
|
|
}
|
|
|
|
HIP_API_CALL(hipDeviceSynchronize());
|
|
HIP_API_CALL(hipDeviceReset());
|
|
|
|
return 0;
|
|
}
|
|
|
|
__global__ void
|
|
transpose(const int* in, int* out, int M, int N)
|
|
{
|
|
__shared__ int tile[shared_mem_tile_dim][shared_mem_tile_dim];
|
|
|
|
int idx = (blockIdx.y * blockDim.y + threadIdx.y) * M + blockIdx.x * blockDim.x + threadIdx.x;
|
|
tile[threadIdx.y][threadIdx.x] = in[idx];
|
|
__syncthreads();
|
|
idx = (blockIdx.x * blockDim.x + threadIdx.y) * N + blockIdx.y * blockDim.y + threadIdx.x;
|
|
out[idx] = tile[threadIdx.x][threadIdx.y];
|
|
}
|
|
|
|
template <typename Tp>
|
|
__global__ void
|
|
test_page_migrate(Tp* data, Tp val)
|
|
{
|
|
int idx = (blockIdx.x * blockDim.x) + threadIdx.x;
|
|
data[idx] += val;
|
|
}
|
|
|
|
__global__ void
|
|
test_kern_large(uint64_t* output)
|
|
{
|
|
uint64_t result = 0;
|
|
int test[4000];
|
|
memset(test, 5, 4000);
|
|
for(int& i : test)
|
|
{
|
|
i = i + 7;
|
|
*output += i;
|
|
result += i;
|
|
}
|
|
*output ^= result;
|
|
*output ^= result;
|
|
}
|
|
|
|
__global__ void
|
|
test_kern_medium(uint64_t* output)
|
|
{
|
|
uint64_t result = 0;
|
|
int test[175];
|
|
memset(test, 5, 175);
|
|
for(int& i : test)
|
|
{
|
|
i = i + 7;
|
|
*output += i;
|
|
result += i;
|
|
}
|
|
*output ^= result;
|
|
*output ^= result;
|
|
}
|
|
|
|
__global__ void
|
|
test_kern_small(uint64_t* output)
|
|
{
|
|
uint64_t result = 0;
|
|
int test[2];
|
|
for(int& i : test)
|
|
{
|
|
i = i + 7;
|
|
*output += i;
|
|
result += i;
|
|
}
|
|
*output ^= result;
|
|
*output ^= result;
|
|
}
|
|
|
|
void
|
|
run(int rank, int tid, int devid, int argc, char** argv)
|
|
{
|
|
auto* stream = hipStream_t{};
|
|
HIP_API_CALL(hipSetDevice(devid));
|
|
HIP_API_CALL(hipStreamCreate(&stream));
|
|
|
|
run_migrate(rank, tid, stream, argc, argv);
|
|
run_scratch(rank, tid, stream, argc, argv);
|
|
run_transpose(rank, tid, stream, argc, argv);
|
|
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
HIP_API_CALL(hipStreamDestroy(stream));
|
|
}
|
|
|
|
void
|
|
run_transpose(int rank, int tid, hipStream_t stream, int argc, char** argv)
|
|
{
|
|
auto* exe_name = ::basename(argv[0]);
|
|
|
|
unsigned int M = 4960 * 2;
|
|
unsigned int N = 4960 * 2;
|
|
if(argc > 2) nitr = atoll(argv[2]);
|
|
if(argc > 3) nsync = atoll(argv[3]);
|
|
|
|
auto_lock_t _lk{print_lock};
|
|
std::cout << "[" << exe_name << "][transpose][" << rank << "][" << tid << "] M: " << M
|
|
<< " N: " << N << std::endl;
|
|
_lk.unlock();
|
|
|
|
std::default_random_engine _engine{std::random_device{}() * (rank + 1) * (tid + 1)};
|
|
std::uniform_int_distribution<int> _dist{0, 1000};
|
|
|
|
size_t size = sizeof(int) * M * N;
|
|
int* inp_matrix = new int[size];
|
|
int* out_matrix = new int[size];
|
|
for(size_t i = 0; i < M * N; i++)
|
|
{
|
|
inp_matrix[i] = _dist(_engine);
|
|
out_matrix[i] = 0;
|
|
}
|
|
int* in = nullptr;
|
|
int* out = nullptr;
|
|
|
|
HIP_API_CALL(hipMalloc(&in, size));
|
|
HIP_API_CALL(hipMalloc(&out, size));
|
|
HIP_API_CALL(hipMemsetAsync(in, 0, size, stream));
|
|
HIP_API_CALL(hipMemsetAsync(out, 0, size, stream));
|
|
HIP_API_CALL(hipMemcpyAsync(in, inp_matrix, size, hipMemcpyHostToDevice, stream));
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
dim3 grid(M / 32, N / 32, 1);
|
|
dim3 block(32, 32, 1); // transpose
|
|
|
|
print_lock.lock();
|
|
printf("[%s][transpose][%i][%i] grid=(%i,%i,%i), block=(%i,%i,%i)\n",
|
|
exe_name,
|
|
rank,
|
|
tid,
|
|
grid.x,
|
|
grid.y,
|
|
grid.z,
|
|
block.x,
|
|
block.y,
|
|
block.z);
|
|
print_lock.unlock();
|
|
|
|
auto t1 = std::chrono::high_resolution_clock::now();
|
|
for(size_t i = 0; i < nitr; ++i)
|
|
{
|
|
transpose<<<grid, block, 0, stream>>>(in, out, M, N);
|
|
check_hip_error();
|
|
if(i % nsync == (nsync - 1)) HIP_API_CALL(hipStreamSynchronize(stream));
|
|
}
|
|
auto t2 = std::chrono::high_resolution_clock::now();
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
HIP_API_CALL(hipMemcpyAsync(out_matrix, out, size, hipMemcpyDeviceToHost, stream));
|
|
double time = std::chrono::duration_cast<std::chrono::duration<double>>(t2 - t1).count();
|
|
float GB = (float) size * nitr * 2 / (1 << 30);
|
|
|
|
print_lock.lock();
|
|
std::cout << "[" << exe_name << "][transpose][" << rank << "][" << tid
|
|
<< "] Runtime of transpose is " << time << " sec\n";
|
|
std::cout << "[" << exe_name << "][transpose][" << rank << "][" << tid
|
|
<< "] The average performance of transpose is " << GB / time << " GBytes/sec"
|
|
<< std::endl;
|
|
print_lock.unlock();
|
|
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
// cpu_transpose(matrix, out_matrix, M, N);
|
|
verify(inp_matrix, out_matrix, M, N);
|
|
|
|
HIP_API_CALL(hipFree(in));
|
|
HIP_API_CALL(hipFree(out));
|
|
|
|
delete[] inp_matrix;
|
|
delete[] out_matrix;
|
|
}
|
|
|
|
void
|
|
run_scratch(int rank, int tid, hipStream_t stream, int, char** argv)
|
|
{
|
|
auto t1 = std::chrono::high_resolution_clock::now();
|
|
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
const auto* exe_name = ::basename(argv[0]);
|
|
|
|
uint64_t* data_ptr = nullptr;
|
|
HIP_API_CALL(hipHostMalloc(&data_ptr, sizeof(uint64_t), 0));
|
|
*data_ptr = 0;
|
|
|
|
test_kern_small<<<1000, 1, 0, stream>>>(data_ptr);
|
|
test_kern_medium<<<1000, 1, 0, stream>>>(data_ptr);
|
|
test_kern_small<<<1000, 1, 0, stream>>>(data_ptr);
|
|
test_kern_large<<<1100, 1, 0, stream>>>(data_ptr);
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
test_kern_small<<<1000, 1, 0, stream>>>(data_ptr);
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
test_kern_medium<<<1000, 1, 0, stream>>>(data_ptr);
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
test_kern_small<<<1000, 1, 0, stream>>>(data_ptr);
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
test_kern_large<<<1100, 1, 0, stream>>>(data_ptr);
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
auto t2 = std::chrono::high_resolution_clock::now();
|
|
double time = std::chrono::duration_cast<std::chrono::duration<double>>(t2 - t1).count();
|
|
|
|
print_lock.lock();
|
|
std::cout << "[" << exe_name << "][scratch][" << rank << "][" << tid
|
|
<< "] Runtime of scratch is " << time << " sec\n";
|
|
print_lock.unlock();
|
|
}
|
|
|
|
void
|
|
run_migrate(int rank, int tid, hipStream_t stream, int, char** argv)
|
|
{
|
|
using data_type = uint64_t;
|
|
constexpr data_type init_v = 1;
|
|
constexpr data_type incr_v = 1;
|
|
|
|
auto t1 = std::chrono::high_resolution_clock::now();
|
|
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
const auto* exe_name = ::basename(argv[0]);
|
|
auto page_data = std::vector<data_type>(1024, 0);
|
|
|
|
HIP_API_CALL(hipHostRegister(
|
|
page_data.data(), page_data.size() * sizeof(data_type), hipHostRegisterDefault));
|
|
|
|
for(auto& itr : page_data)
|
|
itr = init_v;
|
|
|
|
test_page_migrate<<<1, 1024, 0, stream>>>(page_data.data(), incr_v);
|
|
|
|
HIP_API_CALL(hipStreamSynchronize(stream));
|
|
|
|
for(auto& itr : page_data)
|
|
{
|
|
auto diff = (itr - incr_v);
|
|
if(diff != init_v)
|
|
{
|
|
auto msg = std::stringstream{};
|
|
msg << "invalid diff: " << diff << ". expected: " << init_v;
|
|
throw std::runtime_error{msg.str()};
|
|
}
|
|
}
|
|
|
|
HIP_API_CALL(hipHostUnregister(page_data.data()));
|
|
|
|
auto t2 = std::chrono::high_resolution_clock::now();
|
|
double time = std::chrono::duration_cast<std::chrono::duration<double>>(t2 - t1).count();
|
|
|
|
print_lock.lock();
|
|
std::cout << "[" << exe_name << "][migrate][" << rank << "][" << tid
|
|
<< "] Runtime of migrate is " << time << " sec\n";
|
|
print_lock.unlock();
|
|
}
|
|
|
|
namespace
|
|
{
|
|
void
|
|
check_hip_error(void)
|
|
{
|
|
hipError_t err = hipGetLastError();
|
|
if(err != hipSuccess)
|
|
{
|
|
auto_lock_t _lk{print_lock};
|
|
std::cerr << "Error: " << hipGetErrorString(err) << std::endl;
|
|
throw std::runtime_error("hip_api_call");
|
|
}
|
|
}
|
|
|
|
void
|
|
verify(int* in, int* out, int M, int N)
|
|
{
|
|
for(int i = 0; i < 10; i++)
|
|
{
|
|
int row = rand() % M;
|
|
int col = rand() % N;
|
|
if(in[row * N + col] != out[col * M + row])
|
|
{
|
|
auto_lock_t _lk{print_lock};
|
|
std::cout << "mismatch: " << row << ", " << col << " : " << in[row * N + col] << " | "
|
|
<< out[col * M + row] << "\n";
|
|
}
|
|
}
|
|
}
|
|
} // namespace
|